function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const micro = require('micro');
const url = require('url');
const microCors = require('micro-cors');
const denodeify = require('denodeify');
const renderIndex = require('./render-index');
const route = require('./route');

const { json, send } = micro;

const createServer = (feed, broadcast, sse, statsDb, origin = '*') => {
  let storeLog = (() => {
    var _ref = _asyncToGenerator(function* (rawData) {
      const data = Object.assign(rawData, {
        time: Date.now()
      });
      return append(data).then(function () {
        return broadcast.setState(data);
      });
    });

    return function storeLog(_x) {
      return _ref.apply(this, arguments);
    };
  })();

  const append = denodeify(feed.append.bind(feed));
  const cors = microCors({
    allowMethods: ['POST'],
    origin
  });

  return micro(route({
    '*': cors,
    '/': {
      GET: (() => {
        var _ref2 = _asyncToGenerator(function* (req, res) {
          send(res, 200, renderIndex(feed.key.toString('hex')));
        });

        return function GET(_x2, _x3) {
          return _ref2.apply(this, arguments);
        };
      })(),
      POST: (() => {
        var _ref3 = _asyncToGenerator(function* (req, res) {
          if (origin !== '*') {
            if (!req.headers.origin) return send(res, 403, 'Not Allowed');
            const { host, hostname } = url.parse(req.headers.origin);
            if (host !== origin || hostname !== origin) {
              return send(res, 403, 'Not Allowed');
            }
          }
          const body = yield json(req);
          if (!body.event) throw micro.createError(400, '"event" is required');
          if (!body.pathname) body.pathname = 'NO_PATHNAME';
          yield storeLog(body);
          send(res, 204, null);
        });

        return function POST(_x4, _x5) {
          return _ref3.apply(this, arguments);
        };
      })()
    },
    '/_live': {
      GET: (() => {
        var _ref4 = _asyncToGenerator(function* (req, res) {
          sse.addClient(req, res);
        });

        return function GET(_x6, _x7) {
          return _ref4.apply(this, arguments);
        };
      })()
    },
    '/_stats': {
      GET: (() => {
        var _ref5 = _asyncToGenerator(function* (req, res) {
          send(res, 200, statsDb.getAllEvents());
        });

        return function GET(_x8, _x9) {
          return _ref5.apply(this, arguments);
        };
      })()
    }
  }));
};

module.exports = createServer;